// class ColorCreator is used to create any color


import java.awt.*;
import java.applet.Applet;
import java.awt.event.*;


public class ColorCreator extends Panel implements ActionListener, AdjustmentListener
{
// Static variables

  public static final int HORIZONTAL = 1,
                          VERTICAL = 2;
                          
  private static Color defaultColor = Color.green;
  private static int defaultAlignment = ColorCreator.HORIZONTAL;


// Protected members which contain key information

  protected Color currentColor;
  protected int alignment;

  
// Other member variables

  private Panel scrollbarTextFieldPanel,
                scrollbarTextFieldPanels[],
                colorButtonPanel,
                buttonPanel;

  private ColorCanvas colorCanvas;

  private Button buttonArray[];

  private Color purpleColor = new Color(255, 0, 255),
                buttonColors[] = {Color.red,   Color.yellow, Color.blue,
                                  Color.black, Color.gray,   Color.white,
                                  purpleColor, Color.orange, Color.green};
  private String buttonNames[] = {"Red",       "Yellow",     "Blue",
                                  "Black",     "Random",     "White",
                                  "Purple",    "Orange",     "Green"};
  private final int numberOfButtons = buttonColors.length;

  private Scrollbar scrollbarArray[];

  private TextField textFieldArray[];


// Constructors

  public ColorCreator()
  {
    this(ColorCreator.defaultAlignment, ColorCreator.defaultColor);
  }

  public ColorCreator(int alignment)
  {
    this(alignment, ColorCreator.defaultColor);
  }

  public ColorCreator(Color currentColor)
  {
    this(ColorCreator.defaultAlignment, currentColor);
  }

  public ColorCreator(int alignment, Color currentColor)
  {
    createPanels();
    createButtons();
    createTextFieldsScrollbars();
    addComponents();
    setAlignment(alignment);
    update(currentColor);
  }


// public methods

  public Color getCurrentColor()
  {
    return currentColor;
  }

  public void setCurrentColor(Color currentColor)
  {
    this.currentColor = currentColor;
  }

  public int getAlignment()
  {
    return alignment;
  }

  public void setAlignment(int alignment)
  {
    this.alignment = alignment;
    if (alignment == ColorCreator.HORIZONTAL) {
      setLayout(new GridLayout(1,2));
    }
    else {
      setLayout(new GridLayout(2,1));
    }
    validate();
    colorButtonPanel.validate();
  }


// Private and protected methods

  private void createPanels()
  {
    scrollbarTextFieldPanel = new Panel(new GridLayout(3,1));
    scrollbarTextFieldPanels = new Panel[3];
    for (int i = 0; i < 3; i++)
      scrollbarTextFieldPanels[i] = new Panel(new BorderLayout());
      
    colorButtonPanel = new Panel( new GridLayout(1,2) );   // alignment is set in setAlignment
    colorCanvas = new ColorCanvas();
    buttonPanel = new Panel(new GridLayout(3,3));
  }

  private void createButtons()
  {
    buttonArray = new Button[numberOfButtons];
    for (int i = 0; i < numberOfButtons; i++) {
      buttonArray[i] = new Button();
      buttonArray[i].setBackground(buttonColors[i]);
      buttonArray[i].setActionCommand(buttonNames[i]);
      buttonArray[i].addActionListener(this);
      if ( buttonArray[i].getActionCommand() == "Random" )
        buttonArray[i].setLabel("?");
    }
  }

  private void createTextFieldsScrollbars()
  {
    textFieldArray = new TextField[3];
    TextFieldHandler fieldListener = new TextFieldHandler(this);
    scrollbarArray = new Scrollbar[3];
    
    for (int i = 0; i < 3; i++) {
      textFieldArray[i] = new TextField(3);
      textFieldArray[i].setBackground(Color.white);
      textFieldArray[i].addActionListener(fieldListener);

      scrollbarArray[i] = new Scrollbar(Scrollbar.HORIZONTAL, 100, 30, 0, 255+30);
      scrollbarArray[i].setUnitIncrement(1);
      scrollbarArray[i].setBlockIncrement(30);
      scrollbarArray[i].addAdjustmentListener(this);
    }
  }

  private void addComponents()
  {
    add(scrollbarTextFieldPanel);
    add(colorButtonPanel);

    for (int i = 0; i < 3; i++) {
      scrollbarTextFieldPanel.add(scrollbarTextFieldPanels[i]);
      scrollbarTextFieldPanels[i].add(scrollbarArray[i], BorderLayout.CENTER);
      scrollbarTextFieldPanels[i].add(textFieldArray[i], BorderLayout.EAST);
    }

    colorButtonPanel.add(colorCanvas);
    colorButtonPanel.add(buttonPanel);
    for (int i = 0; i < numberOfButtons; i++) {
      buttonPanel.add(buttonArray[i]);
    }
  }

 // Event handling methods.

  public void actionPerformed(ActionEvent e)
  {
    String event = e.getActionCommand();
    for (int i = 0; i < numberOfButtons; i++)
      if (event == buttonNames[i])
        update( buttonColors[i] );
    if (event == "Random")
      update( getRandomColor() );
  }

  public void adjustmentValueChanged(AdjustmentEvent e)
  {
    int colors[] = new int[3];
    for (int i = 0; i < 3; i++) 
      colors[i] = scrollbarArray[i].getValue();
    update(new Color(colors[0], colors[1], colors[2]));
  }

  protected void update(Color color)
  {
    currentColor = color;
    int colors[] = new int[3];
    colors[0] = color.getRed();
    colors[1] = color.getGreen();
    colors[2] = color.getBlue();

    for (int i = 0; i < 3; i++) {
      scrollbarArray[i].setValue(colors[i]);
      textFieldArray[i].setText(String.valueOf(colors[i]));
    }

    colorCanvas.setBackground(currentColor);
    colorCanvas.repaint();

  } // end method update

  protected Color getTextFieldValuesColor()
  {
    int colors[] = new int[3];
    for (int i = 0; i < 3; i++) {
      String string = textFieldArray[i].getText();
      try {
        colors[i] = Integer.parseInt(string);
      }
      catch ( Exception e)
      {
        colors[i] = 0;
      }
      if (colors[i] < 0)
        colors[i] = 0;
      else if (colors[i] > 255)
        colors[i] = 255;
    }
    return ( new Color( colors[0], colors[1], colors[2] ) );
  }

  private int getRandomInt()
  {
    return ( (int) (256*Math.random()) );
  }

  private Color getRandomColor()
  {
    return ( new Color( getRandomInt(), getRandomInt(), getRandomInt() ) );
  }

} // end class ColorCreator



class ColorCanvas extends Canvas  // Used to display current color
{
  public void paint(Graphics g)
  {
    int width = getSize().width,
        height = getSize().height;
    g.setColor(Color.black);
    g.drawRect(0, 0, width-1, height-1);
    g.drawRect(1, 1, width-3, height-3);
  }
}


class TextFieldHandler implements ActionListener
{
  private ColorCreator colorCreator;

  public TextFieldHandler(ColorCreator colorCreator)
  {
    this.colorCreator = colorCreator;
  }

  public void actionPerformed(ActionEvent e)
  {
    Color color = colorCreator.getTextFieldValuesColor();
    colorCreator.update(color);
  }
}




